package check_form;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(check_conf_option_tag check_conf_option_hash check_serv_form_tag check_serv_subform_tag check_serv_option_tag check_serv_option_hash);

#----------------------------------------------------------
# modul check_form.pm
# part of webconf-package for eisfair
# $Id: check_form.pm,v 1.2 2004/02/20 20:54:45 max Exp $
# (c) 2002-2003 by Christian Wuerdig (eisfair@chriswue.de)
#----------------------------------------------------------

use strict;
use CGI::Carp qw(fatalsToBrowser);
use Fcntl ':flock';
use formtools;

our $CHECK_FORM;
our $SERV_LINE_LEN;

no strict "subs";
do "conf.pl";
use strict "subs";

my $form_path = "/var/install/form";
my $prep_path = "/var/install/prep";
my $script_path = "/var/install/servadm";
my $help_path = "/var/install/help";
my $config_path = "/etc/config.d";



###############################################################################
##################### subroutines for config-file formular ####################
###############################################################################



# sub check_conf_option_tag($href, $oref) {{{
sub check_conf_option_tag {
   my ($href, $oref) = @_;
   my ($err, $msg, $tmp, $fmsg) = (0, "", "");

   # fatal errors
   if ($CHECK_FORM eq "yes") {
      if    (!exists($href->{"name"}))                                        { $err = 1; $msg = "tag-option 'name'"; }
      elsif (!exists($href->{"type"}))                                        { $err = 1; $msg = "tag-option 'type'"; }
      elsif ($href->{"type"} !~ /line|yesno|select|ref/i)                     { $err = 1; $msg = "type='line|yesno|select|ref'"; }
      elsif (!exists($href->{"master"}) && !exists($oref->{$href->{"name"}})) { $err = 1; $msg = "an existing variable"; }
      elsif (exists($href->{"master"}) && $href->{"master"} !~ /#/ && !exists($oref->{$href->{"master"}})) { $err = 1; $msg = "an existing master"; }
      elsif (exists($href->{"master"})) {
         if ($href->{"master"} !~ /#/ && $oref->{$href->{"master"}} > 0) {
            $tmp = $href->{"name"};
            $tmp =~ s/#/1/;
            if (!exists($oref->{$tmp})) { $err = 1; $msg = "an existing dependend variable"; }
         } elsif ($href->{"master"} =~ /#/) {
            $tmp = $href->{"master"};
            $tmp =~ s/#/1/;
            if ($oref->{$tmp}) {
               $tmp = $href->{"name"};
               $tmp =~ s/##/1/;
               $tmp =~ s/#/1/;
               if (!exists($oref->{$tmp})) { $err = 1; $msg = "an existing double dependend variable"; }
            }
         }
         
      }
   }
   
   # apply default settings if necessary to avoid problems later
   if (!exists($href->{"name"}))   { $href->{"name"} = ""; }
   if (!exists($href->{"type"}))   { $href->{"type"} = ""; }
   if (!exists($href->{"master"})) { $href->{"master"} = ""; }
   if (!exists($href->{"ro"}))     { $href->{"ro"} = "no"; }

   # alway convert type to lower case
   $href->{"type"} =~ tr/A-Z/a-z/;

   return ($err, $msg, $fmsg);
}
# }}}

# sub check_conf_option_hash($href) {{{
sub check_conf_option_hash {
   my $href = shift;
   my ($err, $msg, $fmsg, $found) = (0, "", "", 0);

   # fatal errors
   if ($CHECK_FORM eq "yes") {
      if ($href->{"type"} =~ /^ref$/i) {
         if    (!exists($href->{"ref"}) && !exists($href->{"file"})) { $err = 1; $msg = "'<ref>' and '<file>'"; $fmsg = "unexpected end of '<option>'"; }
         elsif (!exists($href->{"ref"}))                             { $err = 1; $msg = "'<ref>'"; $fmsg = "unexpected end of '<option>'"; }
         elsif (!exists($href->{"file"}))                            { $err = 1; $msg = "'<file>'"; $fmsg = "unexpected end of '<option>'"; }
         elsif (! -f "$config_path/$href->{file}")   { $err = 1; $msg = "an existing file"; $fmsg = "'$href->{file}'"; }
         else {
            open(FH, "<$config_path/$href->{file}");
            while(<FH>) {
               next if (!/$href->{"ref"}='/);
               $found = 1;
               last;
            }
            close(FH);
            if (!$found) { $err = 1; $msg = "an existing reference"; $fmsg = "unexpected end of '$href->{file}'"; };
         }
      }
   }

   # apply default settings if necessary to avoid problems later
   if (!exists($href->{"match"}))    { $href->{"match"} = ""; }
   if (!exists($href->{"errormsg"})) { $href->{"errormsg"} = "Value '\%v' doesn't match '$href->{match}'"; }
   if (!exists($href->{"val"}))      { $href->{"val"} = ""; }
   if (!exists($href->{"file"}))     { $href->{"file"} = ""; }
   if (!exists($href->{"ref"}))      { $href->{"ref"} = ""; }

   return ($err, $msg, $fmsg);
}
# }}}



###############################################################################
####################### subroutines for service formular ######################
###############################################################################



# sub check_serv_form_tag($href) {{{
sub check_serv_form_tag {
   my $href = shift;
   my ($err, $msg, $fmsg) = (0, "", "");

   # fatal errors
   if ($CHECK_FORM eq "yes") {
      if    (exists($href->{"prep"}) && ! -f "$prep_path/$href->{prep}") { $err = 1; $msg = "existing prepare script"; }
      elsif (exists($href->{"help"}) && ! -f "$help_path/$href->{help}") { $err = 1; $msg = "existing helpfile"; }
   }
   
   # apply default settings if necessary to avoid problems later
   if (!exists($href->{"name"})) { $href->{"name"} = "- no name available -"; }
   if (!exists($href->{"prep"})) { $href->{"prep"} = ""; }
   if (!exists($href->{"help"})) { $href->{"help"} = ""; }

   return ($err, $msg, $fmsg);
}
# }}}

# sub check_serv_subform_tag($href) {{{
sub check_serv_subform_tag {
   my $href = shift;
   my ($err, $msg, $fmsg) = (0, "", "");

   # fatal errors
   if ($CHECK_FORM eq "yes") {
      if    (exists($href->{"target"}) && $href->{"target"} !~ /new|frame|form/i) { $err = 1; $msg = "target='new|frame|form'"; }
      elsif (!exists($href->{"script"}))                                          { $err = 1; $msg = "tag-option 'script'"; }
      elsif ($href->{"script"} =~ /^#(.*)#$/)         { if (! -f "$form_path/$1") { $err = 1; $msg = "an existing formular"; } }
   }

   # apply default settings if necessary to avoid problems later
   if (!exists($href->{"name"}))   { $href->{"name"} = ""; }
   if (!exists($href->{"button"})) { $href->{"button"} = "submit"; }
   if (!exists($href->{"target"})) { $href->{"target"} = "frame"; }
   if (!exists($href->{"script"})) { $href->{"script"} = ""; }

   return ($err, $msg, $fmsg);
}
# }}}

# sub check_serv_option_tag($href) {{{
sub check_serv_option_tag {
   my $href = shift;
   my ($err, $msg, $fmsg) = (0, "", "");

   # fatal errors
   if ($CHECK_FORM eq "yes") {
      if    (!exists($href->{"type"})) { $err = 1; $msg = "tag-option 'type'"; }
      elsif ($href->{"type"} !~ /text|field|line|select|select_m|checkbox|choice/i) {
         $err = 1; $msg = "type='text|field|line|select|select_m|checkbox|choice'"; 
      }
   }
   
   # apply default settings if necessary
   if (!exists($href->{"name"}))  { $href->{"name"} = ""; }
   if (!exists($href->{"vname"})) { $href->{"vname"} = $href->{"name"}; }
   if (!exists($href->{"hname"})) { $href->{"hname"} = ""; }

   # alway convert type to lower case
   $href->{"type"} =~ tr/A-Z/a-z/;

   return ($err, $msg, $fmsg);
}
# }}}

# sub check_serv_option_hash($href) {{{
sub check_serv_option_hash {
   my $href = shift;
   my ($err, $msg, $fmsg) = (0, "", "");

   # apply default settings if necessary
   if (!exists($href->{"align"}))    { $href->{"align"} = "left"; }
   if (!exists($href->{"match"}))    { $href->{"match"} = ""; }
   if (!exists($href->{"errormsg"})) { $href->{"errormsg"} = "Value '\%v' doesn't match '$href->{match}'"; }
   if (!exists($href->{"item"}))     { $href->{"item"} = ""; }
   if (!exists($href->{"text"}))     { $href->{"text"} = ""; }
   
   if    (!exists($href->{"source"})) { $href->{"source"} = ""; $href->{"sourcerange"} = ""; }
   elsif (exists($href->{"source"}) && !exists($href->{"sourcerange"})) {
      if ($href->{"type"} =~ /line/i) { $href->{"sourcerange"} = "1"; }
      else { $href->{"sourcerange"} = "0"; }
   };
   
   if (!exists($href->{"size"})) {
      if    ($href->{"type"} =~ /line/i)            { $href->{"size"} = "$SERV_LINE_LEN"; }
      elsif ($href->{"type"} =~ /field/i)           { $href->{"size"} = "80x10"; }
      elsif ($href->{"type"} =~ /select/i)          { $href->{"size"} = "1"; }
      elsif ($href->{"type"} =~ /checkbox|choice/i) { $href->{"size"} = "h"; }
      else { $href->{"size"} = ""; }
   };

   return ($err, $msg, $fmsg);
}
# }}}

1;
