package formtools;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(get_tag_opt print_head_min print_head printErr init_history add_history print_history);

#----------------------------------------------------------
# modul formtools.pm
# part of webconf-package for eisfair
# $Id: formtools.pm,v 1.4 2004/03/06 13:15:40 chriswue Exp $
# (c) 2002-2004 by Christian Wuerdig (eisfair@chriswue.de)
#----------------------------------------------------------

use strict;
use CGI::Carp qw(fatalsToBrowser);

our $history;

no strict "subs";
do "conf.pl";
use strict "subs";


# get_tag_opt description {{{
##############################################################
# sub get_tag_opt($tag)
# - creates a hash containing the options of a XML-tag
# - returns the complete hash if no hash-key is given
# IN:  $tag    - the tag
#      [$hkey] - key to return (optional)
# OUT:    %hash (optname => content)
#      OR $hash{$hkey}
#
#      note: %hash contains also a pair (eot => "yes|no"),
#            which indicates whether the tag is closed or not
##############################################################
# }}}
# sub get_tag_opt($tag) {{{
sub get_tag_opt {
   my ($tag, $key) = @_;
   my (@opt, %hash, $opt_str, $name, $value);

   # is tag closed?
   $hash{"end_of_tag"} = $tag =~ /^[ \t]*<[a-z]+( [a-z]+="[^"]+")+\/>/ ? 1 : 0;

   # get the tag-options
   while ($tag =~ /^[ \t]*<[a-z]+( [a-z]+="[^"]+")+\/?>[ \t]*(#.*)?\s*$/) {
      $opt_str = substr($tag, $-[1], $+[1] - $-[1], "");
      $opt_str =~ s/^ //;
      push(@opt, $opt_str);
   }

   # split the tag-options and put them in a hash
   foreach (@opt) {
      ($name, $value) = split(/=/, $_);
      $value =~ s/^"|"$//g;
      $hash{"$name"} = $value;
   }
   
   # if no key is given return the whole hash
   if (!$key) { return %hash; }
   else { return exists($hash{"$key"}) ? $hash{"$key"} : ""; }
}
# }}}

# print_head_min description {{{
#######################################
# sub print_head_min($title, $class)
# - prints the head of the HTML-page
# IN:  $title - title of the page
#      $class - CSS-class for the body
# OUT: -
#######################################
# }}}
# sub print_head_min($title, $class) {{{
sub print_head_min {
   my ($title, $class) = @_;

   print "Content-type: text/html\n\n";
   print '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Strict//EN">', "\n";
   print "<html><head>\n";
   print "<title>$title</title>\n";
   print "<link rel=\"stylesheet\" type=\"text/css\" href=\"../CSS/style.css\">\n";
   print "</head><body $class>\n";
}
# }}}

# print_head description {{{
############################################################################
# sub print_head($title, $h1, $class, $no_min)
# - prints a headline with two horizontal lines and the head of a HTML-page
#   if $no_min is _NOT_ set
# IN:  $title    - title of the page
#      $h1       - headline
#      $class    - CSS-class for the body, hr, and h1
#      [$no_min] - sign that only the headline should be printed (optional)
# OUT: -
############################################################################
# }}}
# sub print_head($title, $h1, $class, $no_min) {{{
sub print_head {
   my ($title, $h1, $class, $no_min) = @_;

   if (!$no_min) { print_head_min($title, $class); }
   print "<hr $class>\n";
   print "<h1 $class>$h1</h1>\n";
   print "<hr $class>\n";
}
# }}}

# printErr description {{{
################################################################
# sub printErr($ff, $where, $expected, $found)
# - prints an errormessage
# IN:  $ff       - name of the formfile where the error occured
#      $where    - linenumber
#      $expected - message, what was expected
#      $found    - message, what has been found
# OUT: -
################################################################
# }}}
# sub printErr($ff, $where, $expected, $found) {{{
sub printErr {
   my ($ff, $where, $expected, $found) = @_;
   
   $expected =~ s/</&lt;/g;
   $expected =~ s/>/&gt;/g;
   $found =~ s/</&lt;/g;
   $found =~ s/>/&gt;/g;
   $found =~ s/^(')?[\s]+/$1/;
   
   print "<br><pre style=\"background-color:#FF0000; color:#FFFFFF;";
   print "font-family:'lucidabright', 'Lucida Console', Lucida, Courier, monospace; font-size:110%;\">\n";
   print "<b>Error in formfile '$ff' at line $where!\n";
   print "-&gt; expected $expected\n";
   print "-&gt; found $found\n";
   print "</b></pre><br>\n";
   
   return 1;
}
# }}}

# init_history description {{{
########################################
# sub init_history($entry)
# - initializes the history with $entry
# IN:  $entry - start entry
# OUT: -
########################################
# }}}
# sub init_history($entry) {{{
sub init_history {
   my $entry = shift;
   open(HIST, ">$history") || die("Could not open history, reason: $!\n");
   print HIST "$entry\n";
   close(HIST);
}
# }}}

# add_history description {{{
########################################
# sub add_history($entry)
# - add $entry to the history
# IN:  $entry - history entry
# OUT: -
########################################
# }}}
# sub add_history($entry) {{{
sub add_history {
   my $entry = shift;
   my $last;

   open(HIST, "+>>$history") || die("Could not open history, reason: $!\n");
   seek(HIST, 0, 0);
   while (<HIST>) { $last = $_ if ($_); }
   chop($last);
   print HIST "$entry\n" if ($last ne $entry);

   close(HIST);
}
# }}}

# print_history description {{{
##########################
# sub print_history
# - print the history
# IN:  -
# OUT: -
##########################
# }}}
# sub print_history {{{
sub print_history {
   my $idx = 0;
   my ($name, $ref);
   my $class = 'class="history"';
   my @hist;

   print "<p $class>";
   open(HIST, "<$history") || die("Could not open history, reason: $!\n");
   @hist = <HIST>;
   close(HIST);
   while($idx < $#hist) {
      ($name, $ref) = split(/#/, $hist[$idx], 2);
      if ($idx > 0) {
         print "::";
         print "<a $class href=\"manage_history.pl?$ref+$idx\">$name</a>";
      } else {
         print "<a $class href=\"manage_history.pl?$ref+$idx\" target=\"_parent\">$name</a>";
      }
      $idx++;
   }
   ($name, $ref) = split(/#/, $hist[$#hist], 2);
   print "::" if ($idx);
   print "$name";
   print "</p>\n";
}
# }}}

1;
