package render_config;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(render_config);

#-----------------------------------------------------------------------------
# modul render_config.pm
# part of the webconf-package for EIS/FAIR
# $Id: render_config.pm,v 1.4 2004/02/22 16:39:17 max Exp $
# (c) 2003-2003 by Maximilian Pasternak (maximilian.pasternak@bluescreen.de)
#-----------------------------------------------------------------------------

use strict;

our $VERBOSE;
our $CONF_LINE_LEN;
our $SERV_LINE_LEN;

no strict "subs";
do "conf.pl";
use strict "subs";



##############################################################################
#################### subroutines for config-file          ####################
##############################################################################



#############################################################################
# sub render_config($aref, $href, $cf, $ishelp)
# - writes a new config-file
# IN:  $renderer  - reference to hash with rendering functions
#      $aref      - reference on the form-structure (see parse_confform)
#      $href      - reference on a hash with the options from the config-file
# OUT: -
##############################################################################
sub render_config
{
   my $renderer = shift;
   my $aref     = shift;
   my $href     = shift;

   # get the keys from opt-hash
   my @k = sort keys(%$href);

   #### print the head of the config
   $renderer->{"call"}->("print_header");

   ### formfile found?
   if (@$aref)
   {
      ### loop over all groups
      my $i;
      foreach $i (0 .. $#$aref)
      {
         my $name = $aref->[$i]{"name"};
         if ($name eq "empty")
         {
            $renderer->{"call"}->("print_empty");
         }
         else
         {
            $renderer->{"call"}->("begin_group", $name);
            render_group_options($renderer, $aref->[$i]{"options"}, $href);
            $renderer->{"call"}->("end_group", $name);
         }
      }
   }
   else
   {  
      ### no formfile found, so just print lines
      $renderer->{"call"}->("begin_group", "");
      foreach (@k)
      {
         $renderer->{"print"}->("line", $_, $href->{$_}, "");
      }
      $renderer->{"call"}->("end_group", "");
   }

   ### finish
   $renderer->{"call"}->("print_footer");
   $renderer->{"call"}->("close");
}




sub render_group_options
{
   my $renderer = shift;
   my $options  = shift;
   my $values   = shift;
   my $mymaster = shift || "";
   my $prefix   = shift || "";
   my $replace  = shift || "";

   my $name     = "";
   my $val      = "";
   my $type     = "";

   my $j;
   foreach $j (0 .. $#{$options} )
   {
      my $master = $options->[$j]{"master"} || "";
      next if ("$mymaster" ne "$master");

      if ($options->[$j]{"name"} eq "empty")
      {
         $renderer->{"call"}->("print_empty");
      }
      else
      {
         $name = $options->[$j]{"name"};
         $name =~ s/^$prefix/$replace/  if ($prefix && $name && $replace);
         $val  = $values->{"$name"};
         $type = $options->[$j]{"type"};

         $name = "unknown"   if(!defined($name));
         $val  = ""          if(!defined($val));
         $type = "default"   if(!defined($type));

         $renderer->{"print"}->($type, $name, $val, $options->[$j]);
      }

      if ($options->[$j]{"ismaster"} && $val)
      {
         #print dependent values
         $renderer->{"call"}->("begin_list") unless ($mymaster);

         my $n;
         foreach $n (1 .. $val)
         {
            my $newreplace = $options->[$j]{"ismaster"};
            $newreplace =~ s/^$prefix/$replace/ if ($prefix);
            $newreplace =~ s/#+/$n/;

            render_group_options($renderer, $options, $values,
                                 $options->[$j]{"name"},
                                 $options->[$j]{"ismaster"},
                                 $newreplace);

            $renderer->{"call"}->("next_list") unless ($mymaster);
         }

         $renderer->{"call"}->("end_list") unless ($mymaster);
      }
   }
}

1;

