package syntax;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(decode_string encode_string decode_formstr subst_preregex);

#----------------------------------------------------------
# modul syntax.pm
# part of the webconf-package for eisfair
# last modified: 2003/09/22
# (c) 2002-2003 by Christian Wuerdig (eisfair@chriswue.de)
#----------------------------------------------------------

use strict;
use CGI::Carp qw(fatalsToBrowser);

my $byte = '(?:1?\d{1,2}|2[0-4]\d|25[0-5])';
my $mask = '(?:[12]?\d|3[0-2])';
my $port = '(?:[1-5]?\d{1,4}|6[0-4]\d{3}|65[0-4]\d{2}|655[0-2]\d|6553[0-5])';
my $ip = "$byte(?:\\.$byte){3}";

# Son of RFC1036 {{{
my $comp_start = '[a-z0-9]';
my $comp_rest = '[a-z0-9+_-]';
my $plain_comp = "$comp_start(?:$comp_rest){0,13}";
my $newsgroup = "$plain_comp(?:\\.$plain_comp)*";
# }}}

# RFC 819 {{{
my $char = '[^\\\\,;:@"><()\[\][:cntrl:][:^ascii:]]';
my $qchar = '\\\\[[:ascii:]]|[^\r\n"\\\\[:^ascii:]]';
my $quoted_string = "\"(?:$qchar)+\"";
my $string = "(?:$char)+";
my $address = "#\\d+|\\[$ip\\]|$ip";
my $simple_name = '[[:alpha:]](([[:alnum:]-]+)?[[:alnum:]])?';
my $naming_domain = "(?:$simple_name|$address)";
my $domain = "$naming_domain(?:\\.$naming_domain)*";
my $local_part = "(?:$string|$quoted_string)";
my $mailbox = "$local_part\@$domain";
# }}}

# crontab(5) {{{
my $minute = '(?:[1-5]?\d)';
my $hour = '(?:1?\d|2[0-3])';
my $day_of_month = '(?:[12]?\d|3[01])';
my $month_d = '(?:\d|1[0-2])';
my $month_n = '(?i:jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec)';
my $day_of_week_d = '[0-7]';
my $day_of_week_n = '(?i:mon|tue|wed|thu|fri|sat|sun)';
my $range_min = "$minute(?:-$minute)?";
my $range_hour = "$hour(?:-$hour)?";
my $range_dom = "$day_of_month(?:-$day_of_month)?";
my $range_month = "$month_d(?:-$month_d)?";
my $range_dow = "$day_of_week_d(?:-$day_of_week_d)?";
my $list_min = "$range_min(?:,$range_min)*";
my $list_hour = "$range_hour(?:,$range_hour)*";
my $list_dom = "$range_dom(?:,$range_dom)*";
my $list_month = "$range_month(?:,$range_month)*";
my $list_dow = "$range_dow(?:,$range_dow)*";
my $cron_min = "(?:(?:\\*|$range_min)\\/$minute|(?:\\*|$list_min))";
my $cron_hour = "(?:(?:\\*|$range_hour)\\/$hour|(?:\\*|$list_hour))";
my $cron_dom = "(?:(?:\\*|$range_dom)\\/$day_of_month|(?:\\*|$list_min))";
my $cron_month = "(?:(?:\\*|$range_month)\\/$month_d|(?:\\*|$list_month|$month_n))";
my $cron_dow = "(?:(?:\\*|$range_dow)\\/$day_of_week_d|(?:\\*|$list_dow|$day_of_week_n))";
# }}}

my $name = '(?:[^\\/\x00]|\\\\[^\\/\x00])+';

my %subst_hash;
$subst_hash{"ip"} = "$ip";
$subst_hash{"subnet"} = "$ip\\/(?:$mask|$ip)";
$subst_hash{"domain"} = "$domain";
$subst_hash{"file"} = "\\/?$name(?:\\/$name)*|(?:\\/$name)+";
$subst_hash{"dir"} = "\\/|(?:$subst_hash{file})\\/?";
$subst_hash{"email"} = "$mailbox";
$subst_hash{"hex"} = '0x[[:xdigit:]]+';
$subst_hash{"cron"} = "$cron_min $cron_hour $cron_dom $cron_month $cron_dow(?: .*)?";
$subst_hash{"ng"}= "$newsgroup";
$subst_hash{"port"} = "$port";
$subst_hash{"mode"} = '[0-7]{3,4}|(?:[rwx-]{3}){3}';
$subst_hash{"memsize"} = '\d+[kmgt]?';

# sub decode_string($form_string) {{{
sub decode_string {
   my $string = shift;

   $string =~ tr/+/ /;
   $string =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
   
   return $string;
}
# }}}

# sub encode_string($form_string) {{{
sub encode_string {
   my $string = shift;

   $string =~ s/([^\w ])/"%".unpack("H*", $1)/eg;
   $string =~ tr/ /+/;
   
   return $string;
}
# }}}

# sub decode_formstr($form_string) {{{
sub decode_formstr {
   my $help_str = shift;

   # split formstring
   my @help_list = split(/&/, $help_str);

   foreach (0 .. $#help_list) {
      # decode MIME-encoded chars
      $help_list[$_] =~ s/%0D//g;
      $help_list[$_] = decode_string($help_list[$_]);
   }

   return @help_list;
}
# }}}

# sub subst_preregex(@list) {{{
sub subst_preregex {
   my @help_list = @_;
   my ($key, $value, $subst, $subst_code, $subst_code_e);

   foreach (0 .. $#help_list) {
      while (($key, $value) = each(%subst_hash)) {
         $help_list[$_] =~ s/#$key#/(?:$value)/g;
         $help_list[$_] =~ s/#$key\_e#/(?:$value)?/g;
         $help_list[$_] =~ s/#$key\_s#/^(?:$value)\$/g;
         $help_list[$_] =~ s/#$key\_(se|es)#/^(?:$value)?\$/g;
      }

      if ($help_list[$_] =~ /#user(_e|_s|_se|_es)?#/) {
         my ($ul, $or) = ("", "");
         open(PASSWD, "</etc/passwd") || die "Error: could not open '/etc/passwd'! reason: $!\n";
         my @cont = <PASSWD>;
         close(PASSWD);
         foreach my $entry (@cont) {
            if ($ul ne "") { $or = "|"; }
            $ul .= "$or".substr($entry, 0, index($entry, ":"));
         }
         $help_list[$_] =~ s/#user#/(?:$ul)/g;
         $help_list[$_] =~ s/#user_e#/(?:$ul)?/g;
         $help_list[$_] =~ s/#user_s#/^(?:$ul)\$/g;
         $help_list[$_] =~ s/#user_(se|es)#/^(?:$ul)?\$/g;
      }

      if ($help_list[$_] =~ /#group(_e|_s|_se|_es)?#/) {
         my ($gl, $or) = ("", "");
         open(GROUP, "</etc/group") || die "Error: could not open '/etc/group'! reason: $!\n";
         my @cont = <GROUP>;
         close(GROUP);
         foreach my $entry (@cont) {
            if ($gl ne "") { $or = "|"; }
            $gl .= "$or".substr($entry, 0, index($entry, ":"));
         }
         $help_list[$_] =~ s/#group#/(?:$gl)/g;
         $help_list[$_] =~ s/#group_e#/(?:$gl)?/g;
         $help_list[$_] =~ s/#group_s#/^(?:$gl)\$/g;
         $help_list[$_] =~ s/#group_(se|es)#/^(?:$gl)?\$/g;
      }

      if ($help_list[$_] =~ /#dec_(\d+)(_e|_s|_se|_es)?#/) {
	 my $re;
	 if ($1 > 0) { $re = "\\d{1,$1}"; }
	 else { $re = "\\d+"; }
	 $help_list[$_] =~ s/#dec_(\d+)#/(?:$re)/g;
	 $help_list[$_] =~ s/#dec_(\d+)_e#/(?:$re)?/g;
	 $help_list[$_] =~ s/#dec_(\d+)_s#/^(?:$re)\$/g;
	 $help_list[$_] =~ s/#dec_(\d+)_(se|es)#/^(?:$re)?\$/g;
      }

      if ($help_list[$_] =~ /#dir_exist(_e|_s|_se|_es)?#/) {
         $subst = $subst_hash{"dir"};
         $subst_code = "\$target = \$+; \$yes = -d \$target ? 1 : -1;";
         $subst_code_e = "\$target = \$+; \$yes = -d \$target || \$target eq '' ? 1 : -1;";
         # HIGHLY EXPERIMENTAL extended regex (?{ code })!
         $help_list[$_] =~ s/#dir_exist#/(?:($subst)(?{ $subst_code }))/g;
         $help_list[$_] =~ s/#dir_exist_e#/(?:($subst)?(?{ $subst_code_e }))/g;
         $help_list[$_] =~ s/#dir_exist_s#/((?:^($subst)\$)(?{ $subst_code }))/g;
         $help_list[$_] =~ s/#dir_exist_(se|es)#/(?:^($subst)?\$(?{ $subst_code_e }))/g;
      }

      if ($help_list[$_] =~ /#dir_notexist(_e|_s|_se|_es)?#/) {
         $subst = $subst_hash{"dir"};
         $subst_code = "\$target = \$+; \$yes = -d \$target ? -2 : 1;";
         $subst_code_e = "\$target = \$+; \$yes = \$target ne '' && -d \$target ? -2 : 1;";
         # HIGHLY EXPERIMENTAL extended regex (?{ code })!
         $help_list[$_] =~ s/#dir_notexist#/(?:($subst)(?{ $subst_code }))/g;
         $help_list[$_] =~ s/#dir_notexist_e#/(?:($subst)?(?{ $subst_code_e }))/g;
         $help_list[$_] =~ s/#dir_notexist_s#/(?:^($subst)\$(?{ $subst_code }))/g;
         $help_list[$_] =~ s/#dir_notexist_(se|es)#/(?:^($subst)?\$(?{ $subst_code_e }))/g;
      }

      if ($help_list[$_] =~ /#file_exist(_e|_s|_se|_es)?#/) {
         $subst = $subst_hash{"file"};
         $subst_code = "\$target = \$+; \$yes = -f \$target ? 1 : -3;";
         $subst_code_e = "\$target = \$+; \$yes = -f \$target || \$target eq '' ? 1 : -3;";
         # HIGHLY EXPERIMENTAL extended regex (?{ code })!
         $help_list[$_] =~ s/#file_exist#/(?:($subst)(?{ $subst_code }))/g;
         $help_list[$_] =~ s/#file_exist_e#/(?:($subst)?(?{ $subst_code_e }))/g;
         $help_list[$_] =~ s/#file_exist_s#/(?:^($subst)\$(?{ $subst_code }))/g;
         $help_list[$_] =~ s/#file_exist_(se|es)#/(?:^($subst)?\$(?{ $subst_code_e }))/g;
      }

      if ($help_list[$_] =~ /#file_notexist(_e|_s|_se|_es)?#/) {
         $subst = $subst_hash{"file"};
         $subst_code = "\$target = \$+; \$yes = -f \$target ? -4 : 1;";
         $subst_code_e = "\$target = \$+; \$yes = \$target ne '' && -f \$target ? -4 : 1;";
         # HIGHLY EXPERIMENTAL extended regex (?{ code })!
         $help_list[$_] =~ s/#file_notexist#/(?:($subst)(?{ $subst_code }))/g;
         $help_list[$_] =~ s/#file_notexist_e#/(?:($subst)?(?{ $subst_code_e }))/g;
         $help_list[$_] =~ s/#file_notexist_s#/(?:^($subst)\$(?{ $subst_code }))/g;
         $help_list[$_] =~ s/#file_notexist_(se|es)#/(?:^($subst)?\$(?{ $subst_code_e }))/g;
      }

      $help_list[$_] .= "";
   }

   return @help_list;
}
# }}}

1;
